/* NSUndoManger.m created by veilljf on Fri 18-Dec-1998 */

#import "NSUndoManager.h"
#import <AppKit/NSMenu.h>
#import <AppKit/NSApplication.h>


@interface NSUndoManager (myStepAdditions)

- (void)updateMenuCells;

@end

@implementation NSUndoManager

- init
{
    [super init];
    redoManager = [[NSStackInvocationManager alloc] init];
    redoing = NO;
    return self;
}

- (void)dealloc
{
    [redoManager release];
    [super dealloc];
}

- (void)registerUndoWithTarget:tar
                     selector:(SEL)sel
                       object:ob
    /*" Register an undo invocation."*/
{
    NSMethodSignature *msign = [tar methodSignatureForSelector:sel];
    NSInvocation* invo = [NSInvocation invocationWithMethodSignature:msign];
    [invo setSelector:sel];
    [invo setTarget:tar];
    [invo setArgument:&ob atIndex:2];
    if([invo target] != self) {
        [invo retainArguments];
    }
    if(undoing) {
        // actualy when register method while undoing we should register those as redo.
        [redoManager addEvent:invo];
    } else {
        if(!redoing) {
            [redoManager removeAllActions];
        }
        [self addEvent:invo];
    }
}

- (void)redo
/*"Redo the operation that we just undo.
    *** NOT IMPLEMENTED YET ***
    "*/
{
    redoing = YES;
    [self beginUndoGrouping];
    [redoManager undo];
    [self endUndoGrouping];
    redoing = NO;
}

- (BOOL)canRedo
{
    return [redoManager canUndo];
}

- (void)undo
    /*" Execute last elements of the event stack until you reach the end of the current group."*/
{
    if([self canUndo]) {
        id lastEvent;
        undoing = YES;
        [redoManager beginUndoGrouping];
        lastEvent = [undoEventList lastObject];
        do {
            [lastEvent invoke];
            [undoEventList removeLastObject];
            lastEvent = [undoEventList lastObject];
        } while(!(([lastEvent isEqual:BEGIN_INVOCATION]) && ( undoInvocation == 1)));
        [lastEvent invoke];
        [undoEventList removeLastObject];
        [redoManager endUndoGrouping];
        undoing = NO;
    }
}

- (id) initWithCoder:(NSCoder *)aDecoder
    /*"
    Unarchives a stack instance.
     "*/
{
    [super initWithCoder:aDecoder];
    redoManager = [[aDecoder decodeObject] retain];
    return self;
}

- (void) encodeWithCoder:(NSCoder *)aCoder
    /*"
    Archives a stack instance.
     "*/
{
    [super encodeWithCoder:aCoder];
    [aCoder encodeObject:redoManager];
}

@end


#import "NSStackInvocationManager_private.h"

@implementation NSUndoManager (myStepAdditions)
/*" This category add AppKit context where we have  menu cells interactions. "*/

- undoMenuCell
{
    id result = [[NSApp delegate] undoMenuCell];
    return result;
}

- redoMenuCell
{
    id result = [[NSApp delegate] redoMenuCell];
    return result;
}

- (void)updateMenuCells
/*" this is called to update the menuCells after every menu validation from the NSMenuItem protocol. "*/
{
    id uAction = [self lastGroupName];
    id rAction = [redoManager lastGroupName];
    id uName = (uAction == nil) ? @"" : uAction;
    id rName = (rAction == nil) ? @"" : rAction;

    [[self undoMenuCell] setTitle:[NSString stringWithFormat:@"Undo %@", uName ]];
    [[self redoMenuCell] setTitle:[NSString stringWithFormat:@"Redo %@", rName ]];
}

@end
